#include <stdio.h>
#include <avr/io.h>
#include <util/delay.h>

/* Functie pentru initializarea controller-ului USART */
void USART0_init()
{
    /* setez baud rate la 9600 */
    UBRR0H = 0;
    UBRR0L = 103;

    /* porneste transmitatorul */
    UCSR0B = (1 << TXEN0) | (1 << RXEN0);

    /* seteaza formatul frame-ului: 8 biti de date, 1 biti de stop, fara paritate */
    UCSR0C = (3 << UCSZ00);
}

/*
 * Functie ce primeste un caracter prin USART
 * @return - caracterul primit
 */
char USART0_receive()
{
    /* asteapta cat timp bufferul e gol */
    while(!(UCSR0A & (1<<RXC0)));

    /* returneaza datele din buffer */
    return UDR0;
}

/* configurare porturi folosite */
void set_ports()
{
	/* motoare */
	DDRC |= (1 << PC0);
	DDRC |= (1 << PC1);
	DDRC |= (1 << PC2);
	DDRC |= (1 << PC3);
	/* claxon */
	DDRB |= (1 << PB0);
	PORTB |= (1 << PB0);
	/* faruri */
	DDRA |= (1 << PA0);
	DDRA |= (1 << PA1);
	DDRA |= (1 << PA2);
}

/* setez valorile default pentru motoare si faruri */
void default_values()
{
	PORTC &= ~(1 << PC0);	// fata
	PORTC &= ~(1 << PC1);   // spate
	PORTC &= ~(1 << PC2);	// dreapta
	PORTC &= ~(1 << PC3);	// stanga
	/* pentru faruri */
	PORTA &= ~(1 << PA0);
	PORTA &= ~(1 << PA1);
	PORTA &= ~(1 << PA2);
}

int main()
{
	USART0_init();
	char c;	
	
	/* configurez porturile pe care le folosesc */
	set_ports();
	/* setez valorile default pentru motoare si faruri */
	default_values();

	while(1) {
		
		/* primesc un caracter pentru comanda */
		c = USART0_receive();
		
		/* verific caracterul primit si execut comanda */
		switch(c) {
		/* mers inainte */
		case 'F':
			PORTC |= (1 << PC0);
			PORTC &= ~(1 << PC1);
			PORTC &= ~(1 << PC3);
			PORTC &= ~(1 << PC2);
			break;
		/* mers inapoi */
		case 'B':
			PORTC |= (1 << PC1);
			PORTC &= ~(1 << PC2);
			PORTC &= ~(1 << PC0);
			PORTC &= ~(1 << PC3);
			break;
		/* viraj dreapta */
		case 'R':
			PORTC |= (1 << PC2);
			PORTC &= ~(1 << PC1);
			PORTC &= ~(1 << PC3);
			PORTC &= ~(1 << PC0);
			break;
		/* viraj stanga */
		case 'L':
			PORTC |= (1 <<PC3);
			PORTC &= ~(1 << PC0);
			PORTC &= ~(1 << PC2);
			PORTC &= ~(1 << PC1);
			break;
		/* mers inainte si viraj dreapta */
		case 'I':
			PORTC |= (1 << PC0);
			PORTC &= ~(1 << PC1);
			PORTC |= (1 << PC2);
			PORTC &= ~(1 << PC3);
			break;
		/* mers inainte si viraj stanga */
		case 'G':
			PORTC |= (1 << PC0);
			PORTC &= ~(1 << PC1);
			PORTC |= (1 <<PC3);
			PORTC &= ~(1 << PC2);
			break;
		/* mers inapoi si viraj dreapta */
		case 'J':
			PORTC |= (1 << PC1);
			PORTC &= ~(1 << PC0);
			PORTC |= (1 << PC2);
			PORTC &= ~(1 << PC3);
			break;
		/* mers inapoi si viraj stanga */
		case 'H':
			PORTC |= (1 << PC1);
			PORTC &= ~(1 << PC0);
			PORTC |= (1 <<PC3);
			PORTC &= ~(1 << PC2);
			break;
		
		/* claxon */
		case 'V':	// claxon pornit
			PORTB &= ~(1 << PB0);
			break;
		case 'v':	// claxon oprit
			PORTB |= (1 << PB0);
			break;

		/* faruri */
		case 'W':	// aprinde faruri
			PORTA |= (1 << PA0);
			PORTA |= (1 << PA1);
			PORTA |= (1 << PA2);
			break;
		case 'w':	// stinge faruri
			PORTA &= ~(1 << PA0);
			PORTA &= ~(1 << PA1);
			PORTA &= ~(1 << PA2);
			break;
		
		/* toate motoarele oprite */
		default:
			PORTC &= ~(1 << PC2);
			PORTC &= ~(1 << PC3);
			PORTC &= ~(1 << PC0);
			PORTC &= ~(1 << PC1);
		}
	}
	
	return 0;
}